/*
 *  Header for the Windows Device Driver Interface
 *
 *  Copyright 2007 Marcel Partap <mpartap@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef __WINE_WINDDI_H
#define __WINE_WINDDI_H

#include "wingdi.h"

#ifdef __cplusplus
extern "C" {
#endif

#define DDKAPI WINAPI

/* max number of default hatch patterns */
#define HS_DDI_MAX 6

/* driver version constants */
#define DDI_DRIVER_VERSION_NT4 0x00020000
#define DDI_DRIVER_VERSION_SP3 0x00020003
#define DDI_DRIVER_VERSION_NT5 0x00030000
#define DDI_DRIVER_VERSION_NT5_01 0x000300100

#define DRVQUERY_USERMODE 0x00000001

#define DDI_ERROR 0xffffffff

/* identifiers for function pointers returned by DrvEnableDriver */
#define INDEX_DrvEnablePDEV 0L
#define INDEX_DrvCompletePDEV 1L
#define INDEX_DrvDisablePDEV 2L
#define INDEX_DrvEnableSurface 3L
#define INDEX_DrvDisableSurface 4L
#define INDEX_DrvAssertMode 5L
#define INDEX_DrvOffset 6L
#define INDEX_DrvResetPDEV 7L
#define INDEX_DrvDisableDriver 8L
#define INDEX_DrvCreateDeviceBitmap 10L
#define INDEX_DrvDeleteDeviceBitmap 11L
#define INDEX_DrvRealizeBrush 12L
#define INDEX_DrvDitherColor 13L
#define INDEX_DrvStrokePath 14L
#define INDEX_DrvFillPath 15L
#define INDEX_DrvStrokeAndFillPath 16L
#define INDEX_DrvPaint 17L
#define INDEX_DrvBitBlt 18L
#define INDEX_DrvCopyBits 19L
#define INDEX_DrvStretchBlt 20L
#define INDEX_DrvSetPalette 22L
#define INDEX_DrvTextOut 23L
#define INDEX_DrvEscape 24L
#define INDEX_DrvDrawEscape 25L
#define INDEX_DrvQueryFont 26L
#define INDEX_DrvQueryFontTree 27L
#define INDEX_DrvQueryFontData 28L
#define INDEX_DrvSetPointerShape 29L
#define INDEX_DrvMovePointer 30L
#define INDEX_DrvLineTo 31L
#define INDEX_DrvSendPage 32L
#define INDEX_DrvStartPage 33L
#define INDEX_DrvEndDoc 34L
#define INDEX_DrvStartDoc 35L
#define INDEX_DrvGetGlyphMode 37L
#define INDEX_DrvSynchronize 38L
#define INDEX_DrvSaveScreenBits 40L
#define INDEX_DrvGetModes 41L
#define INDEX_DrvFree 42L
#define INDEX_DrvDestroyFont 43L
#define INDEX_DrvQueryFontCaps 44L
#define INDEX_DrvLoadFontFile 45L
#define INDEX_DrvUnloadFontFile 46L
#define INDEX_DrvFontManagement 47L
#define INDEX_DrvQueryTrueTypeTable 48L
#define INDEX_DrvQueryTrueTypeOutline 49L
#define INDEX_DrvGetTrueTypeFile 50L
#define INDEX_DrvQueryFontFile 51L
#define INDEX_DrvMovePanning 52L
#define INDEX_DrvQueryAdvanceWidths 53L
#define INDEX_DrvSetPixelFormat 54L
#define INDEX_DrvDescribePixelFormat 55L
#define INDEX_DrvSwapBuffers 56L
#define INDEX_DrvStartBanding 57L
#define INDEX_DrvNextBand 58L
#define INDEX_DrvGetDirectDrawInfo 59L
#define INDEX_DrvEnableDirectDraw 60L
#define INDEX_DrvDisableDirectDraw 61L
#define INDEX_DrvQuerySpoolType 62L
#define INDEX_DrvIcmCreateColorTransform 64L
#define INDEX_DrvIcmDeleteColorTransform 65L
#define INDEX_DrvIcmCheckBitmapBits 66L
#define INDEX_DrvIcmSetDeviceGammaRamp 67L
#define INDEX_DrvGradientFill 68L
#define INDEX_DrvStretchBltROP 69L
#define INDEX_DrvPlgBlt 70L
#define INDEX_DrvAlphaBlend 71L
#define INDEX_DrvSynthesizeFont 72L
#define INDEX_DrvGetSynthesizedFontFiles 73L
#define INDEX_DrvTransparentBlt 74L
#define INDEX_DrvQueryPerBandInfo 75L
#define INDEX_DrvQueryDeviceSupport 76L
#define INDEX_DrvReserved1 77L
#define INDEX_DrvReserved2 78L
#define INDEX_DrvReserved3 79L
#define INDEX_DrvReserved4 80L
#define INDEX_DrvReserved5 81L
#define INDEX_DrvReserved6 82L
#define INDEX_DrvReserved7 83L
#define INDEX_DrvReserved8 84L
#define INDEX_DrvDeriveSurface 85L
#define INDEX_DrvQueryGlyphAttrs 86L
#define INDEX_DrvNotify 87L
#define INDEX_DrvSynchronizeSurface 88L
#define INDEX_DrvResetDevice 89L
#define INDEX_DrvReserved9 90L
#define INDEX_DrvReserved10 91L
#define INDEX_DrvReserved11 92L
#define INDEX_LAST 93L

/* constants for EngCreatePalette.ulPaletteMode */
#define PAL_RGB 0x4
#define PAL_BGR 0x8
#define PAL_CMYK 0x10
#define PAL_BITFIELDS 0x2
#define PAL_INDEXED 0x1

/* constants for EngCreateBitmap.iFormat */
#define BMF_1BPP 1L
#define BMF_4BPP 2L
#define BMF_8BPP 3L
#define BMF_16BPP 4L
#define BMF_24BPP 5L
#define BMF_32BPP 6L
#define BMF_4RLE 7L
#define BMF_8RLE 8L
#define BMF_JPEG 9L
#define BMF_PNG 10L

/* constants for EngCreateBitmap.fl */
#define BMF_NOZEROINIT 0x2
#define BMF_TOPDOWN 0x1
#define BMF_USERMEM 0x8

/* bitmap types */
#define STYPE_BITMAP 0L
#define STYPE_DEVBITMAP 3L
#define STYPE_DEVICE 1L

/* constants for EngAssociateSurface.flHooks */
#define HOOK_ALPHABLEND 0x10000
#define HOOK_BITBLT 0x1
#define HOOK_COPYBITS 0x400
#define HOOK_FILLPATH 0x40
#define HOOK_GRADIENTFILL 0x20000
#define HOOK_LINETO 0x100
#define HOOK_MOVEPANNING 0x800
#define HOOK_PAINT 0x10
#define HOOK_PLGBLT 0x4
#define HOOK_STRETCHBLT 0x2
#define HOOK_STRETCHBLTROP 0x2000
#define HOOK_STROKEANDFILLPATH 0x80
#define HOOK_STROKEPATH 0x20
#define HOOK_SYNCHRONIZE 0x1000
#define HOOK_SYNCHRONIZEACCESS 0x4000
#define HOOK_TEXTOUT 0x8
#define HOOK_TRANSPARENTBLT 0x8000

/* constants for CLIPOBJ.iDComplexity */
#define DC_COMPLEX 3
#define DC_RECT 1
#define DC_TRIVIAL 0

/* constants for CLIPOBJ.iFComplexity */
#define FC_COMPLEX 3
#define FC_RECT 1
#define FC_RECT4 2

/* constants for CLIPOBJ.iMode */
#define TC_PATHOBJ 2
#define TC_RECTANGLES 0

/* constants for CLIPOBJ.fjOptions */
#define OC_BANK_CLIP 1

typedef LONG LDEC14;
typedef ULONG ROP4;
typedef ULONG MIX;

typedef LONG_PTR (DDKAPI *PFN)();    /* function pointer */

DECLARE_HANDLE(HBM);
DECLARE_HANDLE(HDEV);
DECLARE_HANDLE(HSURF);
DECLARE_HANDLE(DHSURF);
DECLARE_HANDLE(DHPDEV);
DECLARE_HANDLE(HDRVOBJ);
DECLARE_HANDLE(HSEMAPHORE);

typedef struct _BRUSHOBJ {
    ULONG iSolidColor;
    PVOID pvRbrush;
    FLONG flColorType;
} BRUSHOBJ;

typedef struct _CIECHROMA {
    LDEC14 x;
    LDEC14 y;
    LDEC14 Y;
    } CIECHROMA;

typedef struct _CLIPOBJ {
    ULONG iUniq;
    RECTL rclBounds;
    BYTE iDComplexity;
    BYTE iFComplexity;
    BYTE iMode;
    BYTE fjOptions;
} CLIPOBJ;

typedef struct _COLORINFO {
    CIECHROMA Red;
    CIECHROMA Green;
    CIECHROMA Blue;
    CIECHROMA Cyan;
    CIECHROMA Magenta;
    CIECHROMA Yellow;
    CIECHROMA AlignmentWhite;
    LDEC14 RedGamma;
    LDEC14 GreenGamma;
    LDEC14 BlueGamma;
    LDEC14 MagentaInCyanDye;
    LDEC14 YellowInCyanDye;
    LDEC14 CyanInMagentaDye;
    LDEC14 YellowInMagentaDye;
    LDEC14 CyanInYellowDye;
    LDEC14 MagentaInYellowDye;
} COLORINFO, *PCOLORINFO;

typedef struct _DEVINFO {
    FLONG flGraphicsCaps;
    LOGFONTW lfDefaultFont;
    LOGFONTW lfAnsiVarFont;
    LOGFONTW lfAnsiFixFont;
    ULONG cFonts;
    ULONG iDitherFormat;
    USHORT cxDither;
    USHORT cyDither;
    HPALETTE hpalDefault;
    FLONG flGraphicsCaps2;
} DEVINFO, *PDEVINFO;

typedef struct _DRVFN {
    ULONG iFunc;
    PFN pfn;
} DRVFN, *PDRVFN;

typedef struct _DRVENABLEDATA {
    ULONG iDriverVersion;
    ULONG c;
    DRVFN *pdrvfn;
} DRVENABLEDATA, *PDRVENABLEDATA;

typedef struct _ENG_TIME_FIELDS {
    USHORT usYear;
    USHORT usMonth;
    USHORT usDay;
    USHORT usHour;
    USHORT usMinute;
    USHORT usSecond;
    USHORT usMilliseconds;
    USHORT usWeekday;
} ENG_TIME_FIELDS, *PENG_TIME_FIELDS;

typedef struct _GDIINFO {
    ULONG ulVersion;
    ULONG ulTechnology;
    ULONG ulHorzSize;
    ULONG ulVertSize;
    ULONG ulHorzRes;
    ULONG ulVertRes;
    ULONG cBitsPixel;
    ULONG cPlanes;
    ULONG ulNumColors;
    ULONG flRaster;
    ULONG ulLogPixelsX;
    ULONG ulLogPixelsY;
    ULONG flTextCaps;
    ULONG ulDACRed;
    ULONG ulDACGreen;
    ULONG ulDACBlue;
    ULONG ulAspectX;
    ULONG ulAspectY;
    ULONG ulAspectXY;
    LONG xStyleStep;
    LONG yStyleStep;
    LONG denStyleStep;
    POINTL ptlPhysOffset;
    SIZEL szlPhysSize;
    ULONG  ulNumPalReg;
    COLORINFO ciDevice;
    ULONG ulDevicePelsDPI;
    ULONG ulPrimaryOrder;
    ULONG ulHTPatternSize;
    ULONG ulHTOutputForm;
    ULONG flHTFlags;
    ULONG ulVRefresh;
    ULONG ulBltAlignment;
    ULONG ulPanningHorzRes;
    ULONG ulPanningVertRes;
    ULONG xPanningAlignment;
    ULONG yPanningAlignment;
    ULONG cxHTPat;
    ULONG cyHTPat;
    LPBYTE pHTPatA;
    LPBYTE pHTPatB;
    LPBYTE pHTPatC;
    ULONG flShadeBlend;
    ULONG ulPhysicalPixelCharacteristics;
    ULONG ulPhysicalPixelGamma;
} GDIINFO, *PGDIINFO;

typedef struct _PERBANDINFO {
    BOOL bRepeatThisBand;
    SIZEL szlBand;
    ULONG ulHorzRes;
    ULONG ulVertRes;
} PERBANDINFO, *PPERBANDINFO;

typedef struct _SURFOBJ {
    DHSURF dhsurf;
    HSURF hsurf;
    DHPDEV dhpdev;
    HDEV hdev;
    SIZEL sizlBitmap;
    ULONG cjBits;
    PVOID pvBits;
    PVOID pvScan0;
    LONG lDelta;
    ULONG iUniq;
    ULONG iBitmapFormat;
    USHORT iType;
    USHORT fjBitmap;
} SURFOBJ;

typedef struct _XFORMOBJ {
    ULONG ulReserved;
} XFORMOBJ;

typedef struct _XLATEOBJ {
    ULONG iUniq;
    FLONG flXlate;
    USHORT iSrcType;
    USHORT iDstType;
    ULONG cEntries;
    ULONG *pulXlate;
} XLATEOBJ;

typedef BOOL DDKAPI (*PFN_DrvEnableDriver)(ULONG, ULONG, DRVENABLEDATA*);
typedef VOID DDKAPI (*PFN_DrvDisableDriver)(VOID);
typedef DHPDEV DDKAPI (*PFN_DrvEnablePDEV)(DEVMODEW*, LPWSTR, ULONG, HSURF*, ULONG, ULONG*, ULONG, DEVINFO*, HDEV, LPWSTR, HANDLE);
typedef VOID DDKAPI (*PFN_DrvCompletePDEV)(DHPDEV, HDEV);
typedef BOOL DDKAPI (*PFN_DrvResetPDEV)(DHPDEV, DHPDEV);
typedef VOID DDKAPI (*PFN_DrvDisablePDEV)(DHPDEV);
typedef HSURF DDKAPI (*PFN_DrvEnableSurface)(DHPDEV);
typedef VOID DDKAPI (*PFN_DrvDisableSurface)(DHPDEV);
typedef BOOL DDKAPI (*PFN_DrvQueryDeviceSupport)(SURFOBJ*, XLATEOBJ*, XFORMOBJ*, ULONG, ULONG, PVOID, ULONG, PVOID);
typedef BOOL DDKAPI (*PFN_DrvQueryDriverInfo)(DWORD, PVOID, DWORD, PDWORD);

typedef BOOL DDKAPI (*PFN_DrvEndDoc)(SURFOBJ*, FLONG);
typedef BOOL DDKAPI (*PFN_DrvNextBand)(SURFOBJ*, POINTL*);
typedef ULONG DDKAPI (*PFN_DrvQueryPerBandInfo)(SURFOBJ*, PERBANDINFO*);
typedef BOOL DDKAPI (*PFN_DrvSendPage)(SURFOBJ*);
typedef BOOL DDKAPI (*PFN_DrvStartBanding)(SURFOBJ*, POINTL*);
typedef BOOL DDKAPI (*PFN_DrvStartDoc)(SURFOBJ*, LPWSTR, DWORD);
typedef BOOL DDKAPI (*PFN_DrvStartPage)(SURFOBJ*);

typedef BOOL DDKAPI (*PFN_DrvBitBlt)(SURFOBJ*, SURFOBJ*, SURFOBJ*, CLIPOBJ*, XLATEOBJ*, RECTL*, POINTL*, POINTL*, BRUSHOBJ*, POINTL*, ROP4);
typedef BOOL DDKAPI (*PFN_DrvCopyBits)(SURFOBJ*, SURFOBJ*, CLIPOBJ*, XLATEOBJ*, RECTL*, POINTL*);
/* typedef BOOL DDKAPI (*PFN_DrvStretchBlt)(SURFOBJ*, SURFOBJ*, SURFOBJ*, CLIPOBJ*, XLATEOBJ*, ); */
typedef BOOL DDKAPI (*PFN_DrvLineTo)(SURFOBJ*, CLIPOBJ*, BRUSHOBJ*, LONG, LONG, LONG, LONG, RECTL*, MIX);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __WINE_WINDDI_H */
