#include "printproxydrv.h"
#include <sys/timeb.h>
#define printf putLog
#include "helpfcts.h"

#define buffersize 2048

char szBuffer[buffersize] = "";
char szOrgDLL[MAX_PATH] = "";
char szLogFile[MAX_PATH] = "";

HANDLE hLogFile = INVALID_HANDLE_VALUE;
HINSTANCE hSelfDLL = INVALID_HANDLE_VALUE;
HMODULE hOrigGraphicsDLL = INVALID_HANDLE_VALUE;

DWORD dwWritten = 0;

void putHexDump(const char *memname, LPVOID base, DWORD size) {
#ifndef HEX_PER_LINE
#define HEX_PER_LINE 24
#endif
    LPSTR ptr;
    LPSTR fmt;
    DWORD len;
    DWORD offset = 0;
    len = size - (size & (HEX_PER_LINE -1) );
    fmt =  (len > 99999) ? "%p (0x%05lx/%06ld): " : \
        (len > 65536) ? "%p (0x%05lx/%05ld): " : \
        (len >  9999) ? "%p (0x%04lx/%05ld): " : \
        (len >  4096) ? "%p (0x%04lx/%04ld): " : \
        (len >   999) ? "%p (0x%03lx/%04ld): " : \
        (len >   256) ? "%p (0x%03lx/%03ld): " : \
        (len >    99) ? "%p (0x%02lx/%03ld): " : \
                "%p (0x%02lx/%02ld): ";
    ptr = (LPSTR) base;
    putLog("dumping %s with size of %u Bytes..\r\n", memname, size);
    putLog("%p (size: 0x%lx/%ld)\r\n", ptr, size, size); 
    while (size > 0) {
        offset = ptr - (LPSTR) base;
        putLog(fmt, ptr, offset, offset);
        len = min(size, HEX_PER_LINE);
        while (len > 0) {
            offset++;
            putLog("%02x%s", (int) (*ptr & 0xff), ((offset & 3) == 0) ? " " : "");
            len--;
            ptr++;
        }
        len = min(size, HEX_PER_LINE);
        while (len < HEX_PER_LINE) {
            offset++;
            putLog("  %s", ((offset & 3) == 0) ? " " : "");
            len++;
        }
        putLog(": ");
        len = min(size, HEX_PER_LINE);
        ptr -= len;
        offset = ptr - (LPSTR) base;
        while (len > 0) {
            offset++;
            putLog("%c%s", isprint((int) (*ptr & 0xff)) ? (int) (*ptr & 0xff) :  '.',
                 ((offset & 3) == 0) ? " " : "");
            len--;
            ptr++;
        }
        putLog("\r\n");
        len = min(size, HEX_PER_LINE);
        size -= len;
    }
}

BOOL DLL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
    /* on loading of dll..*/
    if (fdwReason == DLL_PROCESS_ATTACH) {
        /* save instance handle */
        hSelfDLL = hinstDLL;
        /* no further calls shall be made to dll entry point until unloading of last instance */
        DisableThreadLibraryCalls(hinstDLL);
    } else
        /* unload log file when unloading dll */
        if (fdwReason == DLL_PROCESS_DETACH) {
            if (hLogFile != INVALID_HANDLE_VALUE)
                CloseHandle(hLogFile);
        }
    return TRUE;
}

BOOL initLogFile(void) {
    if (hSelfDLL == INVALID_HANDLE_VALUE) return FALSE;
    dwWritten = GetModuleFileName(hSelfDLL, szBuffer, buffersize);
    if (dwWritten > 0) {
        strncpy(szLogFile, szBuffer, dwWritten -4); /* leave out the .dll part */
        strcat(szLogFile, "-GraphicsDLL.log");

        /* get a handle to the log file which is either created or overwritten */
        hLogFile = CreateFile(szLogFile, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
        if (hLogFile == INVALID_HANDLE_VALUE) {
            MessageBox(0, "Creating GraphicsDLL.log failed\r\n", 0, MB_OK|MB_ICONERROR);
            return FALSE;
        }
        SetFilePointer(hLogFile, 0, NULL, FILE_END);
    } else return FALSE;
    return TRUE;
}

BOOL initOrgDLL(void) {
    dwWritten = GetModuleFileName(hSelfDLL, szBuffer, buffersize);
    if (dwWritten > 0) {
        strncpy(szOrgDLL, szBuffer, dwWritten -4); /* again: cut the .dll part */
        strcat(szOrgDLL, "_org.dll");
        /* load the original printer driver graphics dll */
        hOrigGraphicsDLL = LoadLibrary(szOrgDLL);
        if (!hOrigGraphicsDLL) {
            MessageBox(0, "Couldn't load graphics DLL!\r\n", szOrgDLL, MB_OK|MB_ICONERROR);
            return FALSE;
        }

        /* now get the original function addresses */
#define HOOKEXPORTED(x) if (!(p##x = (void*)GetProcAddress(hOrigGraphicsDLL,#x))) { \
                            MessageBox(0, #x, "Couldn't hook function!\r\n", MB_OK|MB_ICONERROR); \
                            return FALSE; \
                        }
        HOOKEXPORTED(DrvEnableDriver);
        HOOKEXPORTED(DrvDisableDriver);
        HOOKEXPORTED(DrvQueryDriverInfo);
    }
        return TRUE;
}

BOOL initProxy(void) { /* returns true if dll and log could be opened */
    BOOL bInitRes = FALSE;

    /* load original dll if not yet loaded */
    if (hOrigGraphicsDLL == INVALID_HANDLE_VALUE) {
        bInitRes = initOrgDLL();
    } else bInitRes = TRUE;

    /* if dll was successfully loaded, open log file for writing */
    if (bInitRes == TRUE) {
        if (hLogFile == INVALID_HANDLE_VALUE) {
            bInitRes = initLogFile();
        } else bInitRes = TRUE;
    }
    return bInitRes;
}

void LogTime(void) {
    struct _timeb time;
    char timestr[20], systime[10];

    /* only continue writing to file if the handle is valid */
    if (hLogFile == INVALID_HANDLE_VALUE) return;
    _strtime(systime);
    _ftime(&time);
    sprintf(timestr, "\r\n%s.%03d:\r\n", systime, time.millitm);
    WriteFile(hLogFile, timestr, 17, &dwWritten, NULL);
}

void putLog(const char *format, ...) {
    va_list argList;

    va_start(argList, format);

    /* only continue writing to file if the handle is valid */
    if (hLogFile == INVALID_HANDLE_VALUE) return;
    dwWritten = vsprintf(szBuffer, format, argList);
    va_end(argList);
    WriteFile(hLogFile, szBuffer, dwWritten, &dwWritten, NULL);
}

BOOL DLL WINAPI DrvEnableDriver(IN ULONG iEngineVersion, IN ULONG cj, OUT DRVENABLEDATA *pded) {
    BOOL bFuncRes;
    int i;
    DRVENABLEDATA tempded;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvEnableDriver(\r\n\tIN ULONG iEngineVersion = 0x%p,\r\n\tIN ULONG cj = %d,\r\n\tOUT DRVENABLEDATA *pded = 0x%p)\r\n", iEngineVersion, cj, pded);

    ZeroMemory(&tempded, sizeof(DRVENABLEDATA));
    if ((bFuncRes = pDrvEnableDriver(iEngineVersion, cj, &tempded))){
        pded->pdrvfn = HeapAlloc(GetProcessHeap(), 0, sizeof(DRVFN)*tempded.c);
        pded->iDriverVersion = tempded.iDriverVersion;
        pded->c = tempded.c;
        for (i=0; i<tempded.c; i++) {
            putLog("tempded.pdrvfn[%d].iFunc=%d (.pfn = 0x%x)\r\n", i, tempded.pdrvfn[i].iFunc, tempded.pdrvfn[i].pfn);
            pded->pdrvfn[i].iFunc = tempded.pdrvfn[i].iFunc;
            switch (tempded.pdrvfn[i].iFunc){
#define HOOK(x) case INDEX_##x: pded->pdrvfn[i].pfn = (void*)GetProcAddress(hSelfDLL, #x); \
                                p##x = (void*)tempded.pdrvfn[i].pfn; putLog("connected %s\r\n", #x); \
                                break;
                HOOK(DrvEnablePDEV);
                HOOK(DrvCompletePDEV);
                HOOK(DrvResetPDEV);
                HOOK(DrvDisablePDEV);
                HOOK(DrvEnableSurface);
                HOOK(DrvDisableSurface);
                HOOK(DrvStartDoc);
                HOOK(DrvEndDoc);
                HOOK(DrvStartPage);
                HOOK(DrvSendPage);
                HOOK(DrvStartBanding);
                HOOK(DrvNextBand);
                HOOK(DrvQueryPerBandInfo);
                HOOK(DrvQueryDeviceSupport);
                HOOK(DrvCopyBits);
                HOOK(DrvBitBlt);
            }
        }
    }
    putLog("return: BOOL %d (pded->c = %d)\r\n", bFuncRes, pded->c);
    return bFuncRes;
}

DHPDEV DLL WINAPI DrvEnablePDEV(IN DEVMODEW *pdm, IN LPWSTR pwszLogAddress, IN ULONG cPat, OUT HSURF *phsurfPatterns, IN ULONG cjCaps, OUT ULONG *pdevCaps, IN ULONG cjDevInfo, OUT DEVINFO *pdi, IN HDEV hdev, IN LPWSTR pwszDeviceName, IN HANDLE hDriver) {
    DHPDEV dhFuncRes;

    if (initProxy() == FALSE) return NULL; /* fail if init functions failed */
    LogTime();
    putLog("call: DHPDEV DrvEnablePDEV(\r\n\tIN DEVMODEW *pdm = 0x%p,\r\n\tIN LPWSTR pwszLogAddress = \"%s\",\r\n\tIN ULONG cPat = %d,\r\n\tOUT HSURF *phsurfPatterns = 0x%p,\r\n\tIN ULONG cjCaps = %d,\r\n\tOUT ULONG *pdevCaps = 0x%p,\r\n\tIN ULONG cjDevInfo = %d,\r\n\tOUT DEVINFO *pdi = 0x%p,\r\n\tIN HDEV hdev = 0x%p,\r\n\tIN LPWSTR pwszDeviceName = \"%s\",\r\n\tIN HANDLE hDriver = 0x%p)\r\n", pdm, strdupWtoA(pwszLogAddress), cPat, phsurfPatterns, cjCaps, pdevCaps, cjDevInfo, pdi, hdev, strdupWtoA(pwszDeviceName), hDriver);
    dhFuncRes = pDrvEnablePDEV(pdm, pwszLogAddress, cPat, phsurfPatterns, cjCaps, pdevCaps, cjDevInfo, pdi, hdev, pwszDeviceName, hDriver);
    putLog("return: DHPDEV 0x%p (with LastError = %d)\r\n", dhFuncRes, GetLastError());
    dumpDevInfo(pdi);
    dumpGdiInfo((GDIINFO*)pdevCaps);
    if (dhFuncRes) putHexDump("DHPDEV", dhFuncRes, 0x200);
    return dhFuncRes;
}

VOID DLL WINAPI DrvCompletePDEV(IN DHPDEV dhpdev, IN HDEV hdev) {
    if (initProxy() == FALSE) return; /* fail if init functions failed */
    LogTime();
    putLog("call: VOID DrvCompletePDEV(\r\n\tIN DHPDEV dhpdev = 0x%p,\r\n\tIN HDEV hdev = 0x%p)\r\n", dhpdev, hdev);
    return pDrvCompletePDEV(dhpdev, hdev);
}

BOOL DLL WINAPI DrvResetPDEV(IN DHPDEV dhpdevOld, IN DHPDEV dhpdevNew){
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvResetPDEV(IN DHPDEV dhpdevOld = 0x%x, IN DHPDEV dhpdevNew = 0x%x)\r\n", dhpdevOld, dhpdevNew);
    bFuncRes = pDrvResetPDEV(dhpdevOld, dhpdevNew);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

VOID DLL WINAPI DrvDisablePDEV(IN DHPDEV dhpdev) {
    if (initProxy() == FALSE) return; /* fail if init functions failed */
    LogTime();
    putLog("call: VOID DrvDisablePDEV(IN DHPDEV dhpdev = 0x%p)\r\n", dhpdev);
    pDrvDisablePDEV(dhpdev);
    return;
}


VOID DLL WINAPI DrvDisableDriver(VOID) {
    if (initProxy() == FALSE) return; /* fail if init functions failed */
    LogTime();
    putLog("call: VOID DrvDisableDriver(VOID)\r\n");
    pDrvDisableDriver();
    return;
}


HSURF DLL WINAPI DrvEnableSurface(IN DHPDEV dhpdev) {
    HSURF hFuncRes;

    if (initProxy() == FALSE) return NULL; /* fail if init functions failed */
    LogTime();
    putLog("call: HSURF DrvEnableSurface(IN DHPDEV dhpdev = 0x%p)\r\n", dhpdev);
    hFuncRes = pDrvEnableSurface(dhpdev);
    putLog("return: HSURF 0x%p\r\n", hFuncRes);
    return hFuncRes;
}


VOID DLL WINAPI DrvDisableSurface(IN DHPDEV dhpdev) {
    if (initProxy() == FALSE) return; /* fail if init functions failed */
    LogTime();
    putLog("call: VOID DrvDisableSurface(IN DHPDEV dhpdev = 0x%p)\r\n", dhpdev);
    pDrvDisableSurface(dhpdev);
    return;
}

BOOL DLL WINAPI DrvQueryDriverInfo(IN DWORD dwMode, OUT PVOID pBuffer, IN DWORD cbBuf, OUT PDWORD pcbNeeded) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvQueryDriverInfo(\r\n\tIN DWORD dwMode = 0x%p,\r\n\tOUT PVOID pBuffer = 0x%p,\r\n\tIN DWORD cbBuf = %d,\r\n\tOUT PDWORD pcbNeeded = %d (*= %d))\r\n", dwMode, pBuffer, cbBuf, pcbNeeded, *pcbNeeded);
    bFuncRes = pDrvQueryDriverInfo(dwMode, pBuffer, cbBuf, pcbNeeded);
    putLog("return: BOOL %d (*pBuffer = %d, *pcbNeeded = %d)\r\n", bFuncRes, *(ULONG*)pBuffer, *pcbNeeded);
    return bFuncRes;
}

BOOL DLL WINAPI DrvQueryDeviceSupport(SURFOBJ *pso, XLATEOBJ *pxlo, XFORMOBJ *pxo, ULONG iType, ULONG cjIn, PVOID pvIn, ULONG cjOut, PVOID pvOut) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvQueryDeviceSupport(\r\n\tSURFOBJ *pso = 0x%p,\r\n\tXLATEOBJ *pxlo = 0x%p,\r\n\tXFORMOBJ *pxo = 0x%p,\r\n\tULONG iType = %d,\r\n\tULONG cjIn = %d,\r\n\tPVOID pvIn = 0x%p,\r\n\tULONG cjOut = %d,\r\n\tPVOID pvOut = 0x%p)\r\n", pso, pxlo, pxo, iType, cjIn, pvIn, cjOut, pvOut);
    bFuncRes = pDrvQueryDeviceSupport(pso, pxlo, pxo, iType, cjIn, pvIn, cjOut, pvOut);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvStartDoc(IN SURFOBJ *pso, IN LPWSTR pwszDocName, IN DWORD dwJobId) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvStartDoc(\r\n\tIN SURFOBJ *pso = 0x%p,\r\n\tIN LPWSTR pwszDocName = %s,\r\n\tIN DWORD dwJobId = %d)\r\n", pso, strdupWtoA(pwszDocName), dwJobId);
    bFuncRes = pDrvStartDoc(pso, pwszDocName, dwJobId);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvEndDoc(IN SURFOBJ *pso, FLONG fl) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvEndDoc(\r\n\tIN SURFOBJ *pso = 0x%p,\r\n\tFLONG fl = %d)\r\n", pso, fl);
    bFuncRes = pDrvEndDoc(pso, fl);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvStartPage(IN SURFOBJ *pso) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvStartPage(IN SURFOBJ *pso = 0x%p)\r\n", pso);
    bFuncRes = pDrvStartPage(pso);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvSendPage(IN SURFOBJ *pso) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvSendPage(IN SURFOBJ *pso = 0x%p)\r\n", pso);
    bFuncRes = pDrvSendPage(pso);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvStartBanding(IN SURFOBJ *pso, IN POINTL *pptl) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvStartBanding(\r\n\tIN SURFOBJ *pso = 0x%p,\r\n\tIN POINTL *pptl = 0x%p)\r\n", pso, pptl);
    bFuncRes = pDrvStartBanding(pso, pptl);
    putLog("return: BOOL %d (pptl->x = %d, pptl->y = %d)\r\n", bFuncRes, pptl->x, pptl->y);
    return bFuncRes;
}

BOOL DLL WINAPI DrvNextBand(IN SURFOBJ *pso, IN POINTL *pptl) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvNextBand(\r\n\tIN SURFOBJ *pso = 0x%p,\r\n\tIN POINTL *pptl = 0x%p)\r\n", pso, pptl);
    bFuncRes = pDrvNextBand(pso, pptl);
    putLog("return: BOOL %d (pptl->x = %d, pptl->y = %d)\r\n", bFuncRes, pptl->x, pptl->y);
    return bFuncRes;
}

ULONG DLL WINAPI DrvQueryPerBandInfo(IN SURFOBJ *pso, IN OUT PERBANDINFO *pbi) {
    ULONG ulFuncRes;

    if (initProxy() == FALSE) return DDI_ERROR; /* fail if init functions failed */
    LogTime();
    putLog("call: ULONG DrvQueryPerBandInfo(\r\n\tIN SURFOBJ *pso = 0x%p,\r\n\tIN OUT PERBANDINFO *pbi = 0x%p)\r\n",  pso, pbi);
    ulFuncRes = pDrvQueryPerBandInfo(pso, pbi);
    putLog("return: ULONG %d\r\n", ulFuncRes);
    return ulFuncRes;
}

BOOL DLL WINAPI DrvBitBlt(IN SURFOBJ *psoTrg, IN SURFOBJ *psoSrc, IN SURFOBJ *psoMask, IN CLIPOBJ *pco, IN XLATEOBJ *pxlo, IN RECTL *prclTrg, IN POINTL *pptlSrc, IN POINTL *pptlMask, IN BRUSHOBJ *pbo, IN POINTL *pptlBrush, IN ROP4 rop4) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvBitBlt(\r\n\tIN SURFOBJ *psoTrg = %p,\r\n\tIN SURFOBJ *psoSrc = %p,\r\n\tIN SURFOBJ *psoMask = %p,\r\n\tIN CLIPOBJ *pco = %p,\r\n\tIN XLATEOBJ *pxlo = %p,\r\n\tIN RECTL *prclTrg = %p,\r\n\tIN POINTL *pptlSrc = %p,\r\n\tIN POINTL *pptlMask = %p,\r\n\tIN BRUSHOBJ *pbo = %p,\r\n\tIN POINTL *pptlBrush = %p,\r\n\tIN ROP4 rop4 = %d)\r\n", psoTrg, psoSrc, psoMask, pco, pxlo, prclTrg, pptlSrc, pptlMask, pbo, pptlBrush, rop4);
    bFuncRes = pDrvBitBlt(psoTrg, psoSrc, psoMask, pco, pxlo, prclTrg, pptlSrc, pptlMask, pbo, pptlBrush, rop4);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

BOOL DLL WINAPI DrvCopyBits(IN SURFOBJ *psoDest, IN SURFOBJ *psoSrc, IN CLIPOBJ *pco, IN XLATEOBJ *pxlo, IN RECTL *prclDest, IN POINTL *pptlSrc) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvCopyBits(\r\n\tIN SURFOBJ *psoDest = %p,\r\n\tIN SURFOBJ *psoSrc = %p,\r\n\tIN CLIPOBJ *pco = %p,\r\n\tIN XLATEOBJ *pxlo = %p,\r\n\tIN RECTL *prclDest = %p,\r\n\tIN POINTL *pptlSrc = %p)\r\n", psoDest, psoSrc, pco, pxlo, prclDest, pptlSrc);
    bFuncRes = pDrvCopyBits(psoDest, psoSrc, pco, pxlo, prclDest, pptlSrc);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}
