#include "printproxyui.h"
#include "helpfcts.h"
#include <sys/timeb.h>

#define buffersize 2048

char szBuffer[buffersize] = "";
char szOrgDLL[MAX_PATH] = "";
char szLogFile[MAX_PATH] = "";

HANDLE hLogFile = INVALID_HANDLE_VALUE;
HINSTANCE hSelfDLL = INVALID_HANDLE_VALUE;
HMODULE hOrigInterfaceDLL = INVALID_HANDLE_VALUE;

DWORD dwWritten = 0;

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
    /* on loading of dll..*/
    if (fdwReason == DLL_PROCESS_ATTACH) {
        /* save instance handle */
        hSelfDLL = hinstDLL;
        /* no further calls shall be made to dll entry point until unloading of last instance */
        DisableThreadLibraryCalls(hinstDLL);
    } else
        /* unload log file when unloading dll */
        if (fdwReason == DLL_PROCESS_DETACH) {
            if (hLogFile != INVALID_HANDLE_VALUE)
                CloseHandle(hLogFile);
        }
    return TRUE;
}

BOOL initLogFile(void) {
    dwWritten = GetModuleFileName(hSelfDLL, szBuffer, buffersize);
    if (dwWritten > 0) {
        strncpy(szLogFile, szBuffer, dwWritten -4); /* leave out the .dll part */
        strcat(szLogFile, "-InterfaceDLL.log");

        /* get a handle to the log file which is either created or overwritten */
        hLogFile = CreateFile(szLogFile, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
        if (hLogFile == INVALID_HANDLE_VALUE) {
            MessageBox(0, "Creating InterfaceDLL.log failed\r\n", 0, MB_OK|MB_ICONERROR);
            return FALSE;
        }
        SetFilePointer(hLogFile, 0, NULL, FILE_END);
    } else return FALSE;
    return TRUE;
}

BOOL initOrgDLL(void) {
    if (hSelfDLL == INVALID_HANDLE_VALUE) return FALSE;
    dwWritten = GetModuleFileName(hSelfDLL, szBuffer, buffersize);
    if (dwWritten > 0) {
        strncpy(szOrgDLL, szBuffer, dwWritten -4); /* again: cut the .dll part */
        strcat(szOrgDLL, "_org.dll");
        /* load the original printer driver interface dll */
        hOrigInterfaceDLL = LoadLibrary(szOrgDLL);
        if (!hOrigInterfaceDLL) {
            MessageBox(0, "Couldn't load interface DLL!\r\n", szOrgDLL, MB_OK|MB_ICONERROR);
            return FALSE;
        }

        /* now get the original function addresses */
#define HOOKEXPORTED(x) if (!(p##x = (void*)GetProcAddress(hOrigInterfaceDLL,#x))) { \
                            MessageBox(0, #x, "Couldn't hook function!\r\n", MB_OK|MB_ICONERROR); \
                            /*return FALSE;*/ \
                        }
        HOOKEXPORTED(DrvConvertDevMode);
        HOOKEXPORTED(DrvDocumentPropertySheets);
        HOOKEXPORTED(DrvDevicePropertySheets);
        HOOKEXPORTED(DrvDeviceCapabilities);
        HOOKEXPORTED(DrvDocumentEvent);
        HOOKEXPORTED(DevQueryPrintEx);
        HOOKEXPORTED(DrvDriverEvent);
        HOOKEXPORTED(DrvPrinterEvent);
        HOOKEXPORTED(DrvQueryColorProfile);
        HOOKEXPORTED(DrvQueryJobAttributes);
/*         HOOKEXPORTED(DrvSplDeviceCaps); */
/*         HOOKEXPORTED(DrvUpgradePrinter); */
    }
        return TRUE;
}

BOOL initProxy(void) { /* returns true if dll and log could be opened */
    BOOL bInitRes = FALSE;

    /* load original dll if not yet loaded */
    if (hOrigInterfaceDLL == INVALID_HANDLE_VALUE) {
        bInitRes = initOrgDLL();
    } else bInitRes = TRUE;

    /* if dll was successfully loaded, open log file for writing */
    if (bInitRes == TRUE) {
        if (hLogFile == INVALID_HANDLE_VALUE) {
            bInitRes = initLogFile();
        } else bInitRes = TRUE;
    }
    return bInitRes;
}

void LogTime(void) {
    struct _timeb time;
    char timestr[20], systime[10];

    /* only continue writing to file if the handle is valid */
    if (hLogFile == INVALID_HANDLE_VALUE) return;
    _strtime(systime);
    _ftime(&time);
    sprintf(timestr, "\r\n%s.%03d:\r\n", systime, time.millitm);
    WriteFile(hLogFile, timestr, 17, &dwWritten, NULL);
}

void putLog(const char *format, ...) {
    va_list argList;

    va_start(argList, format);

    /* only continue writing to file if the handle is valid */
    if (hLogFile == INVALID_HANDLE_VALUE) return;
    dwWritten = vsprintf(szBuffer, format, argList);
    va_end(argList);
    WriteFile(hLogFile, szBuffer, dwWritten, &dwWritten, NULL);
}

DLL BOOL WINAPI DrvConvertDevMode(IN LPTSTR pPrinterName, IN PDEVMODE pdmIn, OUT PDEVMODE pdmOut, OUT PLONG pcbNeeded, IN DWORD fMode) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvConvertDevMode(\r\n\tIN LPTSTR pPrinterName = %s,\r\n\tIN PDEVMODE pdmIn = 0x%p,\r\n\tOUT PDEVMODE pdmOut = 0x%p,\r\n\tOUT PLONG pcbNeeded = 0x%p (*= %d),\r\n\tIN DWORD fMode = %d)\r\n", strdupWtoA(pPrinterName), pdmIn, pdmOut, pcbNeeded, *pcbNeeded, fMode);
    bFuncRes = pDrvConvertDevMode(pPrinterName, pdmIn, pdmOut, pcbNeeded, fMode);
    putLog("return: BOOL %d (*pcbNeeded = %d)\r\n", bFuncRes, *pcbNeeded);
    return bFuncRes;
}

DLL LONG WINAPI DrvDocumentPropertySheets(IN PPROPSHEETUI_INFO pPSUIInfo, IN LPARAM lParam) {
    LONG lFuncRes;

    if (initProxy() == FALSE) return 0; /* fail if init functions failed */
    LogTime();
    putLog("call: LONG DrvDocumentPropertySheets(\r\n\tIN PPROPSHEETUI_INFO pPSUIInfo = 0x%p,\r\n\tIN LPARAM lParam = 0x%p)\r\n", pPSUIInfo, lParam);
    dumpdocpropheader(lParam);
    lFuncRes = pDrvDocumentPropertySheets(pPSUIInfo, lParam);
    putLog("return: LONG %d\r\n", lFuncRes);
    return lFuncRes;
}

DLL INT WINAPI DrvDocumentEvent(IN HANDLE hPrinter, IN HDC hdc, IN int iEsc, IN ULONG cbIn, IN PVOID pvIn, IN ULONG cbOut, OUT PVOID pvOut) {
    INT iFuncRes;

    if (initProxy() == FALSE) return DOCUMENTEVENT_FAILURE; /* fail if init functions failed */
    LogTime();
    putLog("call: INT DrvDocumentEvent(\r\n\tIN HANDLE hPrinter = 0x%p,\r\n\tIN HDC hdc = 0x%p,\r\n\tIN int iEsc = %d,\r\n\tIN ULONG cbIn = %d,\r\n\tIN PVOID pvIn = 0x%p,\r\n\tIN ULONG cbOut = %d,\r\n\tOUT PVOID pvOut = 0x%p)\r\n", hPrinter,hdc, iEsc, cbIn, pvIn, cbOut, pvOut);
    iFuncRes = pDrvDocumentEvent(hPrinter, hdc, iEsc, cbIn, pvIn, cbOut, pvOut);
    putLog("return: INT %d\r\n", iFuncRes);
    return iFuncRes;
}

DLL DWORD WINAPI DrvDeviceCapabilities(IN HANDLE hPrinter, IN PWSTR pDeviceName, IN WORD iDevCap, OUT VOID *pvOutput, IN PDEVMODE pDevMode) {
    DWORD dFuncRes;

    if (initProxy() == FALSE) return GDI_ERROR; /* fail if init functions failed */
    LogTime();
    putLog("call: DWORD DrvDeviceCapabilities(\r\n\tIN HANDLE hPrinter = 0x%p,\r\n\tIN PWSTR pDeviceName = %s,\r\n\tIN WORD iDevCap = %d,\r\n\tOUT VOID *pvOutput = 0x%p,\r\n\tIN PDEVMODE pDevMode = 0x%p)\r\n", hPrinter, strdupWtoA(pDeviceName), iDevCap, pvOutput, pDevMode);
    dFuncRes = pDrvDeviceCapabilities(hPrinter, pDeviceName, iDevCap, pvOutput, pDevMode);
    putLog("return: DWORD 0x%p\r\n", dFuncRes);
    return dFuncRes;
}

DLL LONG WINAPI DrvDevicePropertySheets(IN PPROPSHEETUI_INFO pPSUIInfo, IN LPARAM lParam) {
    LONG lFuncRes;

    if (initProxy() == FALSE) return 0; /* fail if init functions failed */
    LogTime();
    putLog("call: LONG DrvDevicePropertySheets(\r\n\tIN PPROPSHEETUI_INFO pPSUIInfo = 0x%p,\r\n\tIN LPARAM lParam = 0x%p)\r\n", pPSUIInfo, lParam);
    lFuncRes = pDrvDevicePropertySheets(pPSUIInfo, lParam);
    putLog("return: LONG %d\r\n", lFuncRes);
    return lFuncRes;
}

DLL BOOL WINAPI DevQueryPrintEx(IN PDEVQUERYPRINT_INFO pDQPInfo) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DevQueryPrintEx(IN PDEVQUERYPRINT_INFO pDQPInfo = 0x%p)\r\n", pDQPInfo);
    bFuncRes = pDevQueryPrintEx(pDQPInfo);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

DLL BOOL WINAPI DrvDriverEvent(IN DWORD dwDriverEvent, IN DWORD dwLevel, IN LPBYTE pDriverInfo, IN LPARAM lParam) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvDriverEvent(\r\n\tIN DWORD dwDriverEvent = %d,\r\n\tIN DWORD dwLevel = %d,\r\n\tIN LPBYTE pDriverInfo = 0x%p,\r\n\tIN LPARAM lParam = 0x%p)\r\n", dwDriverEvent, dwLevel, pDriverInfo, lParam);
    bFuncRes = pDrvDriverEvent(dwDriverEvent, dwLevel, pDriverInfo, lParam);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

DLL BOOL WINAPI DrvPrinterEvent(IN LPWSTR pPrinterName, IN INT DriverEvent, IN DWORD Flags, IN LPARAM lParam) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvPrinterEvent(\r\n\tIN LPWSTR pPrinterName = %s,\r\n\tIN INT DriverEvent = %d,\r\n\tIN DWORD Flags = %d,\r\n\tIN LPARAM lParam = 0x%p)\r\n", strdupWtoA(pPrinterName), DriverEvent, Flags, lParam);
    bFuncRes = pDrvPrinterEvent(pPrinterName, DriverEvent, Flags, lParam);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

DLL BOOL WINAPI DrvQueryColorProfile(IN HANDLE hPrinter, IN PDEVMODEW pDevmode, IN ULONG ulQueryMode, OUT VOID *pvProfileData, IN OUT ULONG *pcbProfileData, OUT FLONG *pflProfileData) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: DrvQueryColorProfile(\r\n\tIN HANDLE hPrinter = 0x%p,\r\n\tIN PDEVMODEW pDevmode = 0x%p,\r\n\tIN ULONG ulQueryMode = %d,\r\n\tOUT VOID *pvProfileData = 0x%p,\r\n\tIN OUT ULONG *pcbProfileData = 0x%p,\r\n\tOUT FLONG *pflProfileData = 0x%p)\r\n", hPrinter, pDevmode, ulQueryMode, pvProfileData, pcbProfileData, pflProfileData);
    bFuncRes = pDrvQueryColorProfile(hPrinter, pDevmode, ulQueryMode, pvProfileData, pcbProfileData, pflProfileData);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

DLL BOOL WINAPI DrvQueryJobAttributes(IN HANDLE hPrinter, IN PDEVMODE pDevmode, IN DWORD dwLevel, IN LPBYTE lpAttributeInfo) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvQueryJobAttributes(\r\n\tIN HANDLE hPrinter = 0x%p,\r\n\tIN PDEVMODE pDevmode = 0x%p,\r\n\tIN DWORD dwLevel = %d,\r\n\tIN LPBYTE lpAttributeInfo = 0x%p)\r\n", hPrinter, pDevmode, dwLevel, lpAttributeInfo);
    bFuncRes = pDrvQueryJobAttributes(hPrinter, pDevmode, dwLevel, lpAttributeInfo);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

DLL DWORD WINAPI DrvSplDeviceCaps(IN HANDLE hPrinter, IN LPTSTR pwDeviceName, IN WORD DeviceCap, OUT VOID *pvOutput, IN DWORD cchBuf, IN DEVMODE *pDM) {
    DWORD dwFuncRes;

    if (initProxy() == FALSE) return GDI_ERROR; /* fail if init functions failed */
    LogTime();
    putLog("call: DWORD DrvSplDeviceCaps(\r\n\tIN HANDLE hPrinter = 0x%p,\r\n\tIN LPTSTR pwDeviceName = %s,\r\n\tIN WORD DeviceCap = %d,\r\n\tOUT VOID *pvOutput = 0x%p,\r\n\tIN DWORD cchBuf = %d,\r\n\tIN DEVMODE *pDM = 0x%p)\r\n", hPrinter, strdupWtoA(pwDeviceName), DeviceCap, pvOutput, cchBuf, pDM);
    dwFuncRes = pDrvSplDeviceCaps(hPrinter, pwDeviceName, DeviceCap, pvOutput, cchBuf, pDM);
    putLog("return: DWORD 0x%p\r\n", dwFuncRes);
    return dwFuncRes;
}

DLL BOOL WINAPI DrvUpgradePrinter(IN DWORD Level, IN LPBYTE pDriverUpgradeInfo) {
    BOOL bFuncRes;

    if (initProxy() == FALSE) return FALSE; /* fail if init functions failed */
    LogTime();
    putLog("call: BOOL DrvUpgradePrinter(\r\n\tIN DWORD Level = %d,\r\n\tIN LPBYTE pDriverUpgradeInfo = 0x%p)\r\n", Level, pDriverUpgradeInfo);
    bFuncRes = pDrvUpgradePrinter(Level, pDriverUpgradeInfo);
    putLog("return: BOOL %d\r\n", bFuncRes);
    return bFuncRes;
}

